#include "V4.h"
#include "V3.h"
#include <iostream.h>
#include <math.h>

#define RD  57.29578       /* RAD TO DEG CONVERSION */
#define DR  0.0174532925199/* DEG TO RAD CONVERSION */

ClassImp(V4)

// 4-VECTOR
V4::V4(){
  vv.SetV3(0.,0.,0.);
  tt=0.;
}
V4::V4(V3 v ,double t){
  SetV4(v,t);
}
V4::V4(double t,V3 v){
  SetV4(v,t);
}

V4::V4(double x,double y ,double z,double t):vv(x,y,z){
  tt=t;
}
V4::~V4(){
}
//set the vector if the V3 and 4th component are given
void V4::SetV4(V3 v,double t){
  vv.SetV3(v.GetX(),v.GetY(),v.GetZ());
  SetT(t);
}
void V4::SetV4(double x,double y ,double z,double t){
vv.SetX(x);
vv.SetY(y);
vv.SetZ(z);
tt=t;
}
//set single coordinates
void V4::SetX(double x){
vv.SetX(x);
}
void V4::SetY(double y){
vv.SetY(y);
}
void V4::SetZ(double z){
vv.SetZ(z);
}
void V4::SetT(double t){
tt=t;
}
// void V4::SetSpace(double x,double y,double z){ 
//   vv.SetV3(x ,y ,z );
// }
//set spatial part
void V4::SetV(V3 v){ 
  vv.SetV3(v.GetX() ,v.GetY(),v.GetZ() );
}
//print 4-vector
void V4::Print(){
  cout<<"t="<<tt<<" ";
  vv.Print();
}
//get sigle coordinates
double V4::GetX(){
return vv.GetX();
}
double V4::GetY(){
return vv.GetY();
}
double V4::GetZ(){
return vv.GetZ();
}
double V4::GetT(){
return tt;
}
//get spatial part
V3 V4::GetV(){
return vv;
}
//modulus (if mod2 is negative it returns -sqrt(asb(mod2)) )
double V4::Mod(){
double mod2=tt*tt-vv.Mod()*vv.Mod();
if(mod2>0)return sqrt(mod2);
  //return sqrt(tt*tt-vv.Mod()*vv.Mod());
else return -sqrt(-mod2);
}
//lorents transformation along a beta in a generic direction - no rotation
V4 V4::Lorentz(V3 beta){
  double b2=beta.Mod()*beta.Mod();
  double vi[4]= {vv.GetX(),vv.GetY(),vv.GetZ(),tt};
  double bet[3]={beta.GetX(),beta.GetY(),beta.GetZ()};
  V4 vecf;
  double vf[4]={0.,0.,0.,0.};;
  double gamma=1./sqrt(1-b2);
  double Lorentz[4][4];
  //Lorentz matrix definition for general transformation without rotation
  Lorentz[3][3]=gamma;
  for(int i=0;i<3;i++){
    Lorentz[3][i]=-gamma*bet[i];
    Lorentz[i][3]=-gamma*bet[i];
  }
  for(int i=0;i<3;i++){
    for(int j=0;j<3;j++){
      Lorentz[i][j]=((gamma-1)*bet[i]*bet[j])/b2;
      if(i==j)Lorentz[i][j]=Lorentz[i][j]+1;
    }
  }
  //  cout<<"\ngamma="<<gamma<<" beta mod="<<beta.Mod()<<" b2="<<b2<<" vi="<<vv.GetX()<<endl;
//   //test
//   for(int i=0;i<4;i++){
//     for(int j=0;j<4;j++){
//         cout<<i<<" "<<j<<" "<<Lorentz[i][j]<<endl;
//     }
//   }
  //boost
  for(int i=0;i<4;i++){
    for(int j=0;j<4;j++){
      vf[i]=vf[i]+Lorentz[i][j]*vi[j];
    }
  }
  vecf.SetX(vf[0]);
  vecf.SetY(vf[1]);
  vecf.SetZ(vf[2]);
  vecf.SetT(vf[3]);

  return vecf;
 
}
//overloading of + and - operator
V4 V4::operator+(V4 obj2){
  V4 temp;
  temp.vv=vv+obj2.vv;
  temp.tt=tt+obj2.tt;
  return temp;
}
V4 V4::operator-(V4 obj2){
  V4 temp;
  temp.vv=vv-obj2.vv;
  temp.tt=tt-obj2.tt;
  return temp;
}
//rotation - the time component remains unchanged
V4 V4::Rotation(double omega,V3 rotax){
V4 vprime;
vprime.vv=vv.Rotation(omega,rotax);
vprime.tt=tt;
return vprime;
}
